from __future__ import annotations

import logging
import pprint
import traceback

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtWidgets import (
    QApplication,
    QButtonGroup,
    QDialog,
    QGridLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QRadioButton,
    QScrollArea,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class ChooseTestWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        label_choosetest = QLabel()
        label_choosetest.setText('<center><font size="6"> Select Test</font></center>')

        next_button = QPushButton("&Next")
        next_button.clicked.connect(self.next_page)
        back_button = QPushButton("&Back")
        back_button.clicked.connect(self.back_page)

        button_box = QHBoxLayout()

        layout = QVBoxLayout()
        info_layout = self.make_layout()

        button_box.addWidget(back_button)
        button_box.addStretch()
        button_box.addWidget(next_button)

        layout.addWidget(label_choosetest)
        layout.addLayout(info_layout)
        layout.addStretch()
        layout.addLayout(self.make_testlist())
        layout.addStretch()
        layout.addLayout(button_box)
        self.setLayout(layout)

        stage = self.parent.info_dict["currentStage"]
        if len(self.parent.test_dict[stage]) == 0:
            msg = f"For the component {self.parent.info_dict['component']}, there are no tests which can be submitted in the current stage \"{self.parent.info_dict['currentStage']}\"."
            raise RuntimeError(msg)

    def next_page(self):
        title = self.select_test()
        stage = self.parent.info_dict["currentStage"]

        testCode = self.parent.test_dict.get(stage).get(title)

        self.parent.receive_testtype(testCode)

    def back_page(self):
        self.parent.back_from_choosetest()

    def make_testlist(self):
        layout = QHBoxLayout()

        radio_box = QGridLayout()

        label_test = QLabel()
        label_test.setText('<center><font size="3"> Test name</font></center>')
        label_uploaded = QLabel()
        label_uploaded.setText(
            '<center><font size="3"> Upload status in localDB</font></center>'
        )

        self.radiobutton_list = []
        self.radio_group = QButtonGroup()
        check = []
        label_detail = []

        radio_box.addWidget(label_test, 0, 0)
        radio_box.addWidget(label_uploaded, 0, 1)

        stage = self.parent.info_dict["currentStage"]

        for i, test_title in enumerate(self.parent.test_dict[stage].keys()):
            check.append(QLineEdit())
            label_detail.append(MyLabel())

            self.radiobutton_list.append(self.make_radiobutton(test_title))
            # hack
            if (
                self.parent.test_dict[stage][test_title] == "OPTICAL"
                and self.parent.atlsn == ""
                and self.parent.VI_grayout_with_emptyATLSN
            ):
                self.radiobutton_list[i].setStyleSheet(
                    "QRadioButton{font: 15pt; color:gray} QRadioButton::indicator { width: 15px; height: 15px; color:gray};"
                )
                self.radiobutton_list[i].setCheckable(False)

            self.radio_group.addButton(self.radiobutton_list[i], i)
            self.check_uploaded(
                self.parent.test_dict[stage][test_title], check[i], label_detail[i]
            )

            if i == 0:
                self.radiobutton_list[i].setChecked(True)

            radio_box.addWidget(self.radiobutton_list[i], i + 1, 0)
            radio_box.addWidget(check[i], i + 1, 1)
            radio_box.addWidget(label_detail[i], i + 1, 2)

        layout.addStretch()
        layout.addLayout(radio_box)
        layout.addStretch()

        log.info("end of make_testlist")

        return layout

    def check_uploaded(self, test_to_check, form_text, _label_text):
        form_text.setAttribute(Qt.WA_TransparentForMouseEvents)
        form_text.setFocusPolicy(Qt.NoFocus)
        form_text.setAlignment(Qt.AlignCenter)
        form_text.setFixedWidth(150)

        if self.parent.isPractice:
            form_text.setStyleSheet("color: rgb black; background-color:linen;")
            status = "Practice"
        else:
            try:
                stage = self.parent.info_dict["currentStage"]
                status = self.parent.qcStatus["QC_results"][stage]
                log.info("check_uploaded(): " + pprint.pformat(test_to_check))
                log.info("check_uploaded(): " + pprint.pformat(status))

                if status == "-1":
                    form_text.setStyleSheet("color: black; background-color:linen;")
                    status = "Already uploaded"
                else:
                    form_text.setStyleSheet("color: red; background-color:linen;")
                    status = "N/A"

                # self.make_subwin(label_text, upload_status)
            except IndexError:
                form_text.setStyleSheet("color: red; background-color:linen;")
                status = "None"
            except Exception:
                form_text.setStyleSheet("color: rgb black; background-color:linen;")
                log.exception(traceback.format_exc())
                status = "Error"
        form_text.setText(status)

    def make_radiobutton(self, label):
        radiobutton = QRadioButton(label)
        radiobutton.setCheckable(True)
        radiobutton.setFocusPolicy(Qt.NoFocus)
        radiobutton.setStyleSheet(
            "QRadioButton{font: 15pt;} QRadioButton::indicator { width: 15px; height: 15px;};"
        )

        try:
            if label == self.parent.info_dict["testType"]:
                radiobutton.setChecked(True)
        except Exception:
            pass
        return radiobutton

    def make_moreinfo(self, upload_status):
        outer = QVBoxLayout()
        buttons = QHBoxLayout()
        glayout = QGridLayout()
        inner = QScrollArea()
        inner.setFixedHeight(90)
        contents = QWidget()

        close_button = QPushButton("&Close")
        buttons.addStretch()
        buttons.addWidget(close_button)

        title_label = QLabel()
        title_label.setText(
            '<font size="5">Number of uploaded : '
            + str(upload_status["ndoc"])
            + "</font>"
        )
        self.make_infotable(glayout, "Upload user", upload_status["user"], 0)
        self.make_infotable(glayout, "Institution", upload_status["address"], 1)
        self.make_infotable(
            glayout, "Uploaded date(UTC)", upload_status["date"].isoformat(), 2
        )

        contents.setLayout(glayout)
        inner.setWidgetResizable(True)
        inner.setWidget(contents)

        outer.addWidget(title_label)
        outer.addStretch()
        if upload_status["ndoc"] > 1:
            section = QHBoxLayout()
            section_label = QLabel()
            section_label.setText('<font size="4">&bull;latest result</font>')
            section.addWidget(section_label)
            section.addStretch()
            outer.addLayout(section)

        outer.addWidget(inner)
        outer.addLayout(buttons)

        widget = QWidget()
        widget.setLayout(outer)
        return widget

    def make_subwin(self, mylabel, upload_status):
        mylabel.setText("<font color=#1A0DAB>&raquo;more info</font>")
        moreinfo = self.make_moreinfo(upload_status)
        subwin = MyPopup(moreinfo)
        mylabel.clicked.connect(subwin.show_win)

    def select_test(self):
        try:
            log.info("select_test(): " + self.radio_group.checkedButton().text())
            return self.radio_group.checkedButton().text()
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please choose test you want to do", QMessageBox.Ok
            )
            return -1

    def make_layout(self):
        layout = QHBoxLayout()
        vlayout = QGridLayout()

        self.make_infotable(
            vlayout, "Serial Number", self.parent.info_dict["component"], 0
        )
        self.make_infotable(
            vlayout, "Component Type", self.parent.info_dict["componentType"], 1
        )
        self.make_infotable(
            vlayout, "Test Stage", self.parent.localDB_dict["currentStage"], 2
        )

        widget = QWidget()
        widget.setLayout(vlayout)
        widget.setObjectName("widget")
        widget.setStyleSheet("#widget{border: 1px solid black;}")

        layout.addStretch()
        layout.addWidget(widget)
        layout.addStretch()

        return layout

    def make_infotable(self, Grid_layout, str_label, str_info, i):
        label_text = QLabel()
        label_text.setText('<font size="4">' + str_label + "</font>")
        label_sep = QLabel()
        label_sep.setText('<font size="4">:</font>')
        info_text = QLabel()
        info_text.setText('<font size="4">' + str_info + "</font>")

        Grid_layout.addWidget(label_text, i, 0)
        Grid_layout.addWidget(label_sep, i, 1)
        Grid_layout.addWidget(info_text, i, 2)


class MyPopup(QDialog):
    def __init__(self, widget, parent=None):
        super(QDialog, self).__init__(parent)
        self.parent = parent

        close_button = widget.findChild(QPushButton)
        close_button.clicked.connect(self.finish_win)
        self.set_widget(widget)
        self.setWindowTitle("detail information")

    def init_popup(self, layout):
        self.setLayout(layout)

    def set_windowsize(self, x, y):
        self.setGeometry(0, 0, x, y)

    def scale_window(self, x, y):
        QApplication.processEvents()
        self.set_windowsize(x, y)

    def set_widget(self, widget):
        layout = QHBoxLayout()
        layout.addWidget(widget)
        self.setLayout(layout)

    def show_win(self):
        self.show()

    def finish_win(self):
        self.close()


class MyLabel(QLabel):
    mouse_entered = pyqtSignal()
    mouse_left = pyqtSignal()
    clicked = pyqtSignal()

    def __init__(self, parent=None):
        super(QLabel, self).__init__(parent)

    def enterEvent(self, _event):
        self.mouse_entered.emit()

    def leaveEvent(self, _event):
        self.mouse_left.emit()

    def mouseReleaseEvent(self, _event):
        self.clicked.emit()



from __future__ import annotations

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QComboBox,
    QFormLayout,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QMessageBox,
    QPushButton,
    QVBoxLayout,
    QWidget,
)


class InitialWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QVBoxLayout()
        form_layout = QFormLayout()
        button_box = QHBoxLayout()

        label_text = QLabel()
        label_text.setText(
            '<center><font size="6">Input ATLAS Serial Number</font></center>'
        )
        label_atlsn = QLabel()
        label_atlsn.setText("ATLAS Serial Number:")

        #        self.edit_atlsn = QLineEdit()
        #        self.edit_atlsn.setText(self.parent.atlsn)
        self.edit_atlsn = self.make_comp_combo(self.parent.atlsn)

        Back_button = QPushButton("&Back")
        Back_button.clicked.connect(self.parent.login_localdb)

        Next_button = QPushButton("&Next")
        Next_button.clicked.connect(self.pass_atlsn)

        button_box.addWidget(Back_button)
        button_box.addStretch()
        button_box.addWidget(Next_button)

        form_layout.addRow(label_atlsn, self.edit_atlsn)

        layout.addStretch()
        layout.addWidget(label_text)
        layout.addStretch()
        layout.addLayout(form_layout)
        layout.addLayout(button_box)

        self.setLayout(layout)

    def make_comp_combo(self, initial):
        combo = self.make_combobox(initial)
        combo.addItem("")
        combo.addItems(self.parent.component_list)

        combo.lineEdit().textEdited.connect(
            lambda: self.comobo_refresh(combo, combo.currentText())
        )
        return combo

    def comobo_refresh(self, combo, currenttext):
        combo.clear()
        combo.addItem("")
        combo.lineEdit().setText(currenttext)
        if currenttext != "":
            new_list = [
                element
                for element in self.parent.component_list
                if currenttext.upper() in str(element).upper()
            ]
            combo.addItems(new_list)
        else:
            combo.addItems(self.parent.component_list)

    def make_combobox(self, initial):
        combo = QComboBox()
        LE = QLineEdit()
        combo.setLineEdit(LE)
        combo.lineEdit().setText(initial)
        #        combo.lineEdit().setCompleter(None)
        if initial == "TBA":
            LE.setReadOnly(True)
            #            combo.lineEdit().setReadOnly(True)
            combo.setStyleSheet("color: black; background-color:linen;")
            combo.setFocusPolicy(Qt.NoFocus)

        return combo

    def pass_atlsn(self):
        try:
            self.parent.receive_atlsn(self.edit_atlsn.currentText())
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Invalid Serial Number", QMessageBox.Ok
            )



from __future__ import annotations

import logging

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.parylene_properties import initial_Parylene_prop

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__()
        self.parent = parent
        self.initial_wid = None
        self.test_result_dict = None

        self.setWindowTitle("Parylene properties")

        self.result_info = {
            "Parylene_type_list": ["Type-N", "Type-C"],
            "Institution": self.parent.institute_dict,
            "comment": "",
            "batch": "TBA",
            "Parylene_type": "Type-N",
            "Name_of_Masking_Operator": "",
            "Institution_of_Masking_Operator": "",
            "Name_of_Operator_Removing_Mask": "",
            "Institution_of_Operator_Removing_Mask": "",
            "Parylene_thickness_measured_by_vendor": "",
            "Parylene_thickness_measured_by_ITk_Institute": "",
            "Thickness_unit": "um",
        }

    def receive_result(self, result):
        self.result_info["Parylene_type"] = result["Parylene_type"]
        self.result_info["batch"] = result["batch"]
        self.result_info["Name_of_Masking_Operator"] = result[
            "Name_of_Masking_Operator"
        ]
        self.result_info["Name_of_Operator_Removing_Mask"] = result[
            "Name_of_Operator_Removing_Mask"
        ]
        self.result_info["Institution_of_Masking_Operator"] = result[
            "Institution_of_Masking_Operator"
        ]
        self.result_info["Institution_of_Operator_Removing_Mask"] = result[
            "Institution_of_Operator_Removing_Mask"
        ]
        self.result_info["Parylene_thickness_measured_by_vendor"] = result[
            "Parylene_thickness_measured_by_vendor"
        ]
        self.result_info["Parylene_thickness_measured_by_ITk_Institute"] = result[
            "Parylene_thickness_measured_by_ITk_Institute"
        ]
        self.result_info["comment"] = result["comment"]

        self.fill_result()
        self.return_result()

    def fill_result(self):
        self.test_result_dict = {
            "results": {
                "localDB": {
                    "property": {
                        "Parylene_Type": self.result_info["Parylene_type"],
                        "Name_of_Masking_Operator": self.result_info[
                            "Name_of_Masking_Operator"
                        ],
                        "Name_of_Operator_Removing_Mask": self.result_info[
                            "Name_of_Operator_Removing_Mask"
                        ],
                        "Institution_of_Masking_Operator": self.result_info[
                            "Institution_of_Masking_Operator"
                        ],
                        "Institution_of_Operator_Removing_Mask": self.result_info[
                            "Institution_of_Operator_Removing_Mask"
                        ],
                        "Parylene_Batch_Number": self.result_info["batch"],
                    },
                    "Parylene_thickness_measured_by_vendor": self.result_info[
                        "Parylene_thickness_measured_by_vendor"
                    ],
                    "Parylene_thickness_measured_by_ITk_Institute": self.result_info[
                        "Parylene_thickness_measured_by_ITk_Institute"
                    ],
                    "Thickness_unit": self.result_info["Thickness_unit"],
                    "comment": self.result_info["comment"],
                },
                "ITkPD": {
                    "property": {
                        "Parylene_Type": self.result_info["Parylene_type"],
                        "Name_of_Masking_Operator": self.result_info[
                            "Name_of_Masking_Operator"
                        ],
                        "Name_of_Operator_Removing_Mask": self.result_info[
                            "Name_of_Operator_Removing_Mask"
                        ],
                        "Institution_of_Masking_Operator": self.result_info[
                            "Institution_of_Masking_Operator"
                        ],
                        "Institution_of_Operator_Removing_Mask": self.result_info[
                            "Institution_of_Operator_Removing_Mask"
                        ],
                        "Parylene_Batch_Number": self.result_info["batch"],
                    },
                    "Parylene_thickness_measured_by_vendor": self.result_info[
                        "Parylene_thickness_measured_by_vendor"
                    ],
                    "Parylene_thickness_measured_by_ITk_Institute": self.result_info[
                        "Parylene_thickness_measured_by_ITk_Institute"
                    ],
                    "Thickness_unit": self.result_info["Thickness_unit"],
                    "comment": self.result_info["comment"],
                },
                "summary": {
                    "property": {
                        "Parylene_Type": self.result_info["Parylene_type"],
                        "Name_of_Masking_Operator": self.result_info[
                            "Name_of_Masking_Operator"
                        ],
                        "Name_of_Operator_Removing_Mask": self.result_info[
                            "Name_of_Operator_Removing_Mask"
                        ],
                        "Institution_of_Masking_Operator": self.result_info[
                            "Institution_of_Masking_Operator"
                        ],
                        "Institution_of_Operator_Removing_Mask": self.result_info[
                            "Institution_of_Operator_Removing_Mask"
                        ],
                        "Parylene_Batch_Number": self.result_info["batch"],
                    },
                    "Parylene_thickness_measured_by_vendor": self.result_info[
                        "Parylene_thickness_measured_by_vendor"
                    ],
                    "Parylene_thickness_measured_by_ITk_Institute": self.result_info[
                        "Parylene_thickness_measured_by_ITk_Institute"
                    ],
                    "Thickness_unit": self.result_info["Thickness_unit"],
                    "comment": self.result_info["comment"],
                },
            }
        }
        log.info(
            "[Test Result]  "
            + str(
                self.test_result_dict["results"]["localDB"]["property"]["Parylene_Type"]
            )
        )

    ###########################################################################################

    def init_ui(self):
        self.initial_wid = initial_Parylene_prop.InitialWindow(self)
        self.update_widget(self.initial_wid)

    def scale_window(self, x, y):
        self.setGeometry(0, 0, x, y)

    def update_widget(self, w):
        self.scale_window(480, 360)
        self.setCentralWidget(w)
        self.show()

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()

    def return_result(self):
        self.parent.receive_result(self, self.test_result_dict)



from __future__ import annotations

import logging

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.qc_tests.QUAD_MODULE_METROLOGY import initial_metrology

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        #        super(QMainWindow, self).__init__(parent)
        super(QMainWindow, self).__init__()
        self.parent = parent

        self.setGeometry(0, 0, 510, 255)

        self.componentType = "MODULE"
        self.stage = "MODULE/ASSEMBLY"

        self.init_ui()

    def receive_result(self, _result):
        for k, v in _result.get("results").items():
            self.parent.testRun.get("results").get("Measurements").update({k: v})
        self.parent.receive_result(self)

    ############################################################################################
    def init_ui(self):
        self.initial_bare_wid = initial_metrology.InitialWindow(self)
        self.parent.update_widget(self.initial_bare_wid)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()



from __future__ import annotations

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QLabel,
    QLineEdit,
    QScrollArea,
    QTextEdit,
    QWidget,
)


class NotSupportedError(Exception):
    pass


class ConfirmWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        self.parent.confirm_init_common(self, width=500, height=500)

    def back_page(self):
        self.parent.back_to_test()

    def check_json(self):
        self.parent.confirm_json()

    def upload_to_db(self):
        self.parent.upload_to_db()

    def add_info(self, Form_layout, label_str, form_text):
        label = QLabel()
        label.setText(label_str)

        if label_str == "Comment :":
            inner = QTextEdit()
            inner.setText(form_text)
            inner.setReadOnly(True)
            inner.setFocusPolicy(Qt.NoFocus)
            inner.setStyleSheet("background-color : linen; color: black;")

            editor = QScrollArea()
            editor.setWidgetResizable(True)
            editor.setWidget(inner)

        else:
            editor = QLineEdit()
            editor.setText(form_text)
            editor.setReadOnly(True)

        editor.setFocusPolicy(Qt.NoFocus)
        editor.setStyleSheet("background-color : linen; color: black;")
        editor.setFixedWidth(300)
        editor.setMaximumWidth(400)

        Form_layout.addRow(label, editor)

    #################################################################
    def make_layout(self):
        if (
            self.parent.info_dict["componentType"] == "MODULE"
            or self.parent.info_dict["componentType"] == "practice"
        ):
            return self.layout_ModuleQC()
        return self.layout_ModuleQC()

    def layout_ModuleQC(self):
        Form_layout = self.parent.confirm_layout_common(self)

        mass_value = f'{self.parent.testRun["results"]["MASS"]} {self.parent.testRun["results"]["metadata"]["MASS_UNIT"]}'
        acu_value = f'{self.parent.testRun["results"]["property"]["SCALE_ACCURACY"]} {self.parent.testRun["results"]["metadata"]["SCALE_ACCURACY_UNIT"]}'

        self.add_info(Form_layout, "Total weight :", mass_value)
        self.add_info(Form_layout, "Scale accuracy :", acu_value)
        self.add_info(
            Form_layout, "Comment :", self.parent.testRun["results"]["comment"]
        )

        return Form_layout



from __future__ import annotations

import json
import logging
import pprint
import shutil
import subprocess
from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QFileDialog, QMainWindow, QMessageBox

from module_qc_nonelec_gui.dbinterface import localdb_uploader
from module_qc_nonelec_gui.qc_tests.VISUAL_INSPECTION import (
    inspection_win,
    shaping_win,
    splitimg_win,
    summary_win,
    vi_initial_win,
)
from module_qc_nonelec_gui.qc_tests.VISUAL_INSPECTION.functions.cv2_func import (
    cv2,
    img_rotate,
    read_img,
    write_img,
)

log = logging.getLogger(__name__)


class TestWindow(QMainWindow):
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__(parent)
        self.parent = parent

        # other member variables
        self.path_gm = None
        self.checklist_dict = None
        self.initial_wid = None
        self.img_bgr = None
        self.img_h = None
        self.img_w = None
        self.img_d = None
        self.scale = None
        self.n_page = None
        self.shape_img_wid = None
        self.split_img_wid = None
        self.msg_map = None

        self.setGeometry(0, 0, 500, 300)

        self.temp_dir_path = "/var/tmp/module-qc-nonelec-gui/vi"
        try:
            Path(self.temp_dir_path).mkdir(parents=True)
        except FileExistsError:
            shutil.rmtree(self.temp_dir_path)
            Path(self.temp_dir_path).mkdir(parents=True)

        # window title
        self.setWindowTitle("Visual Inspection GUI")

        # list and dict of anomalies
        self.anomaly_dic = {}
        self.comment_dic = {}
        self.img_dic = {}

        # component and user information
        self.atlsn = self.parent.info_dict["component"]
        self.type_name = self.parent.info_dict["componentType"]
        # self.original_institution = self.parent.info_dict["institution"]
        # self.current_location = self.parent.info_dict["currentLocation"]
        if self.type_name in ["MODULE", "PCB", "BARE_MODULE"]:
            self.stage = self.parent.info_dict["currentStage"]
            self.inspector = self.parent.db_user
        else:
            self.stage = ""
            self.inspector = ""

        # load configuration file
        log.info(f"type_name = {self.type_name}")
        log.info(f"stage = {self.stage}")

        stage_alt = self.stage.replace("/", "__")

        if self.type_name in ["MODULE", "PCB", "BARE_MODULE"]:
            self.json_default = str(
                Path(__file__).parent / f"config/config_{self.type_name}_default.json"
            )
            self.json_config = str(
                Path(__file__).parent
                / f"config/config_{self.type_name}_{stage_alt}.json"
            )

        with Path(self.json_default).open(encoding="utf-8") as f:
            self.config = json.load(f)
        with Path(self.json_config).open(encoding="utf-8") as f:
            self.custom = json.load(f)

            for key in [
                "frontside_goldenmodule_path",
                "backside_goldenmodule_path",
                "ntile",
                "nsplit",
                "backside",
            ]:
                if key in self.custom:
                    self.config[key] = self.custom[key]

            if "checklist" in self.custom:
                for side in ["front", "back"]:
                    if side in self.custom["checklist"]:
                        for flag in ["Yellow", "Red"]:
                            if flag in self.custom["checklist"][side]:
                                self.config["checklist"][side][flag] = (
                                    self.custom["checklist"][side][flag]
                                    + self.config["checklist"][side][flag]
                                )

        if self.config.get("backside") is True:
            QMessageBox.warning(
                self,
                "Warning",
                "Back-side inspection is required in this stage: please make sure that the photograph is already recorded at this point, otherwise you will be required to re-do the front-side inspection again!",
            )

        log.info(pprint.pformat(self.config))

        self.origImgPath = {"front": "", "back": ""}
        self.mode = "front"

        # checked page list
        self.rev = 0
        self.tot_page = int(self.config["ntile"])
        self.nsplit = int(self.config["nsplit"])
        self.page_checked = []
        for i in range(self.tot_page):
            self.page_checked.insert(i, False)

    def init_ui(self):
        # check the presence of golden modules

        try:
            self.path_gm = str(
                Path(__file__).parent
                / self.config[f"{self.mode}side_goldenmodule_path"]
            )

        except Exception:
            QMessageBox.warning(
                self,
                "Warning",
                f"{self.mode.capitalize()}-side Golden module path is undefined in the config file!",
            )

        try:
            with (Path(self.path_gm).parent / ".version").open(encoding="utf-8") as f:
                version_info = f.read()

                if version_info.find("2023-09-04") < 0:
                    msg = "Golden module version outdated"
                    raise Exception(msg)

        except Exception:
            log.exception("missing the golden image file")

            while True:
                msgBox = QMessageBox(self)
                msgBox.setTextFormat(Qt.RichText)
                msgBox.setText(
                    'Golden module image bank needs to be deployed.<br /><br />[Step-1] Download the following zip file: <br /><a href="https://cernbox.cern.ch/s/PAMujVUmRUni0um">https://cernbox.cern.ch/s/PAMujVUmRUni0um</a><br /><br />[Step-2] Indicate the path of the zip file to this GUI.'
                )
                msgBox.setStandardButtons(QMessageBox.Ok)
                msgBtn = msgBox.button(QMessageBox.Ok)
                msgBtn.setText("Yes, downloaded the zip file")
                msgBox.exec()

                dlg = QFileDialog(self)
                dlg.setFileMode(QFileDialog.FileMode.ExistingFiles)
                dlg.setNameFilter("ZIP files (*.zip)")
                dlg.setViewMode(QFileDialog.Detail)

                filename = None

                try:
                    dlg.show()
                    if dlg.exec_():
                        filename = dlg.selectedFiles()[0]
                        break
                except Exception:
                    log.error("fail in dlg.exec_()")
                    continue

            subprocess.run(
                f"cp {filename} /var/tmp; cd /var/tmp; unzip {filename.split('/')[-1]}",
                shell=True,
                check=False,
            )

            shutil.rmtree(str(Path(self.path_gm).parent))

            subprocess.run(
                "mv -f /var/tmp/golden_module {}; rm -rf /var/tmp/golden_module*".format(
                    str(Path(self.path_gm).parent.parent)
                ),
                shell=True,
                check=False,
            )

            with Path(self.path_gm + "/main_img.jpg").open(encoding="utf-8") as f:
                pass

            QMessageBox.warning(
                self,
                "Information",
                "The latest golden module image bank was deployed.",
            )

        for index, _check in enumerate(self.page_checked):
            self.page_checked[index] = False

        self.anomaly_dic = {}
        self.comment_dic = {}
        self.img_dic = {}

        # check list
        self.checklist_dict = {}

        checklist = self.config["checklist"].get(self.mode)

        for tile in range(36):
            if checklist:
                self.checklist_dict[str(tile)] = checklist

        self.initial_wid = vi_initial_win.InitialWindow(self)
        self.parent.update_widget(self.initial_wid)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def update_img(self, img):
        self.rev = self.rev + 1
        path = f"{self.temp_dir_path}/img_{self.mode}_{self.rev}.jpg"
        write_img(img, path)
        log.info(f"dumped image {path}")
        self.img_bgr, self.img_h, self.img_w, self.img_d = read_img(path)

    def undo_img(self):
        if self.rev == 0:
            self.rev = self.rev
        else:
            self.rev = self.rev - 1
        path = f"{self.temp_dir_path}/img_{self.mode}_{self.rev}.jpg"
        self.img_bgr, self.img_h, self.img_w, self.img_d = read_img(path)

    def load_img(self, mode="front"):
        options = QFileDialog.Options()
        inputFileName, _ = QFileDialog.getOpenFileName(
            self,
            "Open File",
            "",
            "Images (*.bmp *.dib *.pbm *.pgm *.ppm *.sr *.ras *.jpeg *.jpg *.jpe *.jp2 *.png *.tiff *.tif);;Any files (*)",
            options=options,
        )
        if not inputFileName:
            log.info("image is not chosen")
        else:
            self.statusBar().showMessage(inputFileName)
            log.info(inputFileName)

            self.img_bgr, self.img_h, self.img_w, self.img_d = read_img(inputFileName)

            self.mode = mode

            self.origImgPath[
                self.mode
            ] = f"{self.temp_dir_path}/img_{self.mode}_org.jpg"
            write_img(self.img_bgr, self.origImgPath[self.mode])

            if self.img_h > 1000 and self.img_w > 1000:
                self.scale = 10
            else:
                self.scale = 1
            self.n_page = 0
            log.info("OpenCV image read success.")

            self.rev = 0

            self.shape_img()

    def shape_img(self):
        log.info("shape_img()")
        img, h, w, d = read_img(self.origImgPath.get(self.mode))

        stage = self.parent.info_dict.get("currentStage", "")
        log.info(f"shape_img(): stage = {stage}")

        # if "BAREMODULE" in stage:
        #    if self.mode == "front":
        #        self.shape_img_wid = shaping_win.QuadFrontTrimmer(  # FIXME default QuadFrontTrimmer deosn't work for Bare Modules
        #            self, {}
        #        )
        #    elif self.mode == "back":
        #        self.shape_img_wid = shaping_win.QuadBackTrimmer(self, {})
        if "MODULE" in stage:
            if self.mode == "front":
                self.shape_img_wid = shaping_win.QuadFrontTrimmer(self, {})
            elif self.mode == "back":
                self.shape_img_wid = shaping_win.QuadBackTrimmer(self, {})

        elif "PCB" in stage:
            if self.mode == "front":
                self.shape_img_wid = shaping_win.QuadFrontTrimmer(
                    self, {"CropRange": 31.0}
                )
            elif self.mode == "back":
                self.shape_img_wid = shaping_win.QuadPCBBackTrimmer(
                    self,
                    {
                        "CropRange": 31.0,
                        "Brightness": -50,
                        "Contrast": 2.0,
                        "InitialBlur": 8,
                        "InitialParam2": 35,
                    },
                )

        self.parent.update_widget(self.shape_img_wid)

    def rotate_img(self):
        try:
            img, h, w, d = read_img(
                f"{self.temp_dir_path}/img_{self.mode}_{self.rev}.jpg"
            )
        except Exception:
            img, h, w, d = read_img(self.origImgPath.get(self.mode))

        img, h, w, d = img_rotate(img)
        self.update_img(img)

    def split_img(self):
        self.split_img_wid = splitimg_win.SplitImageWindow(self)
        self.parent.update_widget(self.split_img_wid)

    def inspection(self):
        # Guiding dialog
        self.msg_map = {
            "BAREMODULERECEPTION": "Check carefully for defect at the edges and cornners of both sensor and FE chips.",
            "MODULE/ASSEMBLY": "Check carefully:\n * Glue must not reach the edge of sensor/PCB;\n * Glue does not infiltrate the pads or back-side of the sensor",
            "MODULE/WIREBONDING": "Check carefully:\n * All wires match the wire bond map;\n * No shorts due to bad bonding.",
            "MODULE/PARYLENE_MASKING": "Check carefully:\n * Masking of pickup points;\n * Masking of pigtail connectors;\n * Back-side masking",
            "MODULE/PARYLENE_COATING": "Check carefully:\n * Quality of parylene coating",
            "MODULE/PARYLENE_UNMASKING": 'Check carefully:\n * Has the parylene coating penetrated the pickup and strain relief or connector areas?\n * Are there any areas where de-lamination of the parylene layer is seen that may result in contamination of the module?\n * Is the back of the module free of parylene such that only small "witness marks" are visible on the edges of the ASICs?',
            "MODULE/WIREBOND_PROTECTION": "",
            "MODULE/THERMAL_CYCLES": "",
            "PCB_RECEPTION": "Check carefully the central part for defects, including scratches, surface contamination and other damage.",
            "PCB_POPULATION": "Check carefully the central part for defects, especially soldering problems.",
            "PCB_CUTTING": "",
            "PCB_RECEPTION_MODULE_SITE": "",
        }

        if self.n_page == 0:
            QMessageBox.information(
                self,
                "Note",
                "\n\n".join(
                    [
                        self.parent.info_dict.get("currentStage", ""),
                        self.msg_map.get(self.parent.info_dict.get("currentStage", "")),
                        "Key-binds:\n * X: Checkout Tile\n * F: Next Page\n * B: Previous Page\n * S: Switch Target/Reference",
                    ]
                ),
            )

        inspection_wid = inspection_win.InspectionWindow(self)
        self.parent.update_widget(inspection_wid)

    def make_result(self):
        path_target = f"{self.temp_dir_path}/main_{self.mode}_img.jpg"
        cv2.imwrite(path_target, self.img_bgr)

        oid_target = str(
            localdb_uploader.jpeg_formatter(self.parent.localDB, path_target)
        )

        for page, path_jpeg in self.img_dic.items():
            self.img_dic[page] = str(
                localdb_uploader.jpeg_formatter(self.parent.localDB, path_jpeg)
            )

        self.parent.testRun["results"]["metadata"][
            f"{self.mode}_defects"
        ] = self.anomaly_dic
        self.parent.testRun["results"]["metadata"][
            f"{self.mode}_comments"
        ] = self.comment_dic
        self.parent.testRun["results"]["metadata"][f"{self.mode}_image"] = oid_target
        self.parent.testRun["results"]["metadata"][
            f"{self.mode}_defect_images"
        ] = self.img_dic
        # Fill in operator and instrument properties (PCB only)
        if self.type_name == "PCB":
            self.parent.testRun["results"]["property"]["OPERATOR"] = self.inspector
            self.parent.testRun["results"]["property"]["INSTRUMENT"] = self.instrument

        if self.config.get("backside") is True and self.mode == "front":
            self.mode = "back"
            self.init_ui()

        else:
            self.return_result()

    def go_to_summary(self, sub):
        sub.hide()
        summary_wid = summary_win.SummaryWindow(self)
        self.parent.update_widget(summary_wid)

    def return_result(self):
        self.parent.receive_result(self)



from __future__ import annotations

import logging

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QButtonGroup,
    QGroupBox,
    QHBoxLayout,
    QLabel,
    QMessageBox,
    QPushButton,
    QRadioButton,
    QVBoxLayout,
    QWidget,
)

log = logging.getLogger(__name__)


class ContinueWindow(QWidget):
    def __init__(self, parent=None):
        super(QWidget, self).__init__()
        self.parent = parent

        layout = QVBoxLayout()
        yn_box = QHBoxLayout()
        yn_hbox = QHBoxLayout()
        where_box = QHBoxLayout()
        where_vbox = QVBoxLayout()
        button_box = QHBoxLayout()

        label_text = QLabel()
        label_text.setText(
            '<center><font size="6">Choose the options you want</font></center>'
        )

        button = QPushButton("&Continue")
        button.clicked.connect(self.restart_test)
        fin_button = QPushButton("&Finish")
        fin_button.clicked.connect(self.finishGUI)

        button_box.addWidget(fin_button)
        button_box.addStretch()
        button_box.addWidget(button)

        raddio_yes = self.make_radiobutton("Yes")
        raddio_yes.setChecked(True)
        raddio_no = self.make_radiobutton("No")
        self.yn_radio_group = QButtonGroup()
        self.yn_radio_group.addButton(raddio_yes, 0)
        self.yn_radio_group.addButton(raddio_no, 1)

        yn_hbox.addWidget(raddio_yes)
        yn_hbox.addWidget(raddio_no)

        yn_box.addStretch()
        yn_box.addLayout(yn_hbox)
        yn_box.addStretch()

        yn_group = QGroupBox("Will you inspect as the same user?")
        yn_group.setStyleSheet("QGroupBox { font-size: 15px;font-weight: bold;} ")
        yn_group.setLayout(yn_box)

        raddio_test = self.make_radiobutton("same component")
        raddio_test.setChecked(True)
        raddio_comp = self.make_radiobutton("different component")
        self.where_radio_group = QButtonGroup()
        self.where_radio_group.addButton(raddio_test, 0)
        self.where_radio_group.addButton(raddio_comp, 1)

        where_vbox.addWidget(raddio_test)
        where_vbox.addWidget(raddio_comp)
        where_box.addStretch()
        where_box.addLayout(where_vbox)
        where_box.addStretch()

        where_group = QGroupBox("Which component will you inspect?")
        where_group.setStyleSheet("QGroupBox { font-size: 15px;font-weight: bold;} ")
        where_group.setLayout(where_box)

        layout.addWidget(label_text)
        layout.addStretch()
        layout.addWidget(yn_group)
        layout.addStretch()
        layout.addWidget(where_group)
        layout.addStretch()
        layout.addLayout(button_box)

        self.setLayout(layout)

    def make_radiobutton(self, label):
        radiobutton = QRadioButton(label)
        radiobutton.setCheckable(True)
        radiobutton.setFocusPolicy(Qt.NoFocus)
        radiobutton.setStyleSheet(
            "QRadioButton{font: 12pt;} QRadioButton::indicator { width: 12px; height: 12px;};"
        )
        return radiobutton

    def finishGUI(self):
        self.parent.finish_GUI()

    def restart_test(self):
        try:
            log.info(
                "[Next Inspection] same user : "
                + self.yn_radio_group.checkedButton().text()
            )
            log.info(
                "[Next Inspection] target    : "
                + self.where_radio_group.checkedButton().text()
            )
            self.parent.restart(
                self.yn_radio_group.checkedId(), self.where_radio_group.checkedId()
            )
        except Exception:
            QMessageBox.warning(
                None, "Warning", "Please check each radiobutton.", QMessageBox.Ok
            )



from __future__ import annotations

from PyQt5.QtWidgets import QMainWindow

from module_qc_nonelec_gui.dbinterface import localdb_uploader
from module_qc_nonelec_gui.qc_tests.THERMAL_CYCLING import initial_Thermal


class TestWindow(QMainWindow):
    ############################################################################################
    def __init__(self, parent=None):
        super(QMainWindow, self).__init__()
        self.parent = parent
        self.test_result_dict = None
        self.initial_wid = None

        self.setWindowTitle("Thermal Cycling")

        self.result_info = {
            "comment": "",
            "Machine": "",
            "Temp_log_path": "",
            "Humi_log_path": "",
            "Temp_min_value": "",
            "Temp_max_value": "",
            "Temp_unit": "degC",
            "N_cycle": "",
            "Cycle_speed_value": "",
            "Cycle_speed_unit": "K/min",
        }

    def receive_result(self, result):
        self.result_info["Machine"] = result["Machine"]
        self.result_info["Temp_log_path"] = result["Temp_log_path"]
        self.result_info["Humi_log_path"] = result["Humi_log_path"]
        self.result_info["Temp_min_value"] = result["Temp_min_value"]
        self.result_info["Temp_max_value"] = result["Temp_max_value"]
        self.result_info["N_cycle"] = result["N_cycle"]
        self.result_info["Cycle_speed_value"] = result["Cycle_speed_value"]
        self.result_info["comment"] = result["comment"]

        self.result_info["Temp_log"] = str(
            localdb_uploader.jpeg_formatter(
                self.parent.localDB, self.result_info["Temp_log_path"]
            )
        )
        self.result_info["Humi_log"] = str(
            localdb_uploader.jpeg_formatter(
                self.parent.localDB, self.result_info["Humi_log_path"]
            )
        )

        self.fill_result()
        self.return_result()

    def fill_result(self):
        self.test_result_dict = {
            "results": {
                "localDB": {
                    "property": {
                        "Machine": self.result_info["Machine"],
                        "Temp_min_value": self.result_info["Temp_min_value"],
                        "Temp_max_value": self.result_info["Temp_max_value"],
                        "N_cycle": self.result_info["N_cycle"],
                        "Cycle_speed_value": self.result_info["Cycle_speed_value"],
                        "Cycle_speed_unit": self.result_info["Cycle_speed_unit"],
                    },
                    "Temperature_log": self.result_info["Temp_log"],
                    "Humidity_log": self.result_info["Humi_log"],
                    "Temp_unit": self.result_info["Temp_unit"],
                    "comment": self.result_info["comment"],
                },
                "ITkPD": {
                    "property": {
                        "Machine": self.result_info["Machine"],
                        "Temp_min_value": self.result_info["Temp_min_value"],
                        "Temp_max_value": self.result_info["Temp_max_value"],
                        "N_cycle": self.result_info["N_cycle"],
                        "Cycle_speed_value": self.result_info["Cycle_speed_value"],
                        "Cycle_speed_unit": self.result_info["Cycle_speed_unit"],
                    },
                    "Temperature_log": self.result_info["Temp_log"],
                    "Humidity_log": self.result_info["Humi_log"],
                    "Temp_unit": self.result_info["Temp_unit"],
                    "comment": self.result_info["comment"],
                },
                "summary": {
                    "property": {
                        "Machine": self.result_info["Machine"],
                        "Temp_min_value": self.result_info["Temp_min_value"],
                        "Temp_max_value": self.result_info["Temp_max_value"],
                        "N_cycle": self.result_info["N_cycle"],
                        "Cycle_speed_value": self.result_info["Cycle_speed_value"],
                        "Cycle_speed_unit": self.result_info["Cycle_speed_unit"],
                    },
                    "Temperature_log": self.result_info["Temp_log"],
                    "Humidity_log": self.result_info["Humi_log"],
                    "Temp_log_path": self.result_info["Temp_log_path"],
                    "Humi_log_path": self.result_info["Humi_log_path"],
                    "Temp_unit": self.result_info["Temp_unit"],
                    "comment": self.result_info["comment"],
                },
            }
        }

    ###########################################################################################

    def init_ui(self):
        self.initial_wid = initial_Thermal.InitialWindow(self)
        self.parent.update_widget(self.initial_wid)

    def close_and_return(self):
        self.close()
        self.parent.back_from_test()

    def back_page(self):
        self.parent.init_ui()

    def back_window(self):
        self.parent.receive_backpage()

    def call_another_window(self, window):
        self.hide()
        window.init_ui()

    def return_result(self):
        self.parent.receive_result(self, self.test_result_dict)
